@@ -1,4 +1,4 @@
-/* $Id: freebob_streaming.c 446 2007-10-15 18:54:13Z pieterpalmers $ */
+/* $Id: freebob_streaming.c 452 2008-03-10 14:01:07Z pieterpalmers $ */
 
 /*
  *   FreeBoB Streaming API
@@ -38,6 +38,7 @@
 #include "freebob_debug.h"
 #include "thread.h"
 #include "messagebuffer.h"
+#include "cycletimer.h"
 
 #include <signal.h>
 #include <unistd.h>
@@ -150,11 +151,11 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 		return NULL;
 	}
 
+#ifndef EXCLUDE_SAMPLERATE_SUPPORT
 	/* Try and set the samplerate
- 	 * This should be done after device discovery
+	 * This should be done after device discovery
 	 * but before reading the bus description as the device capabilities can change
 	 */
-
     if(options.node_id > -1) {
         if (freebob_set_samplerate(dev->fb_handle, options.node_id, options.sample_rate) != 0) {
             freebob_destroy_handle(dev->fb_handle);
@@ -180,6 +181,7 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 	    }
 
 	}
+#endif
 
 	/* Read the connection specification
  	 */
@@ -1819,6 +1821,9 @@ void * freebob_iso_packet_iterator(void *arg)
 					
 					assert(connection);
 					
+					// reset the dropped counter every time we iterate
+					connection->status.dropped = 0;
+
 					err = raw1394_loop_iterate (connection->raw_handle);
 					
 					// detect underruns on the connection
@@ -1827,6 +1832,13 @@ void * freebob_iso_packet_iterator(void *arg)
 						underrun_detected=TRUE;
 						break; // we can exit as the underrun handler will flush the buffers anyway
 					}
+
+					// only detect dropped packets after things have settled (1 sec settling)
+					if (connection->status.packets > 8000 && connection->status.dropped > 0) {
+						printError("Dropped packets on connection %d\n", connection->spec.id);
+						underrun_detected=TRUE;
+						break; // we can exit as the underrun handler will flush the buffers anyway
+					}
 				
 					if (err == -1) {
 						printError ("possible raw1394 error: %s\n", strerror (errno));
@@ -2118,6 +2130,7 @@ iso_master_receive_handler(raw1394handle_t handle, unsigned char *data,
         unsigned int dropped)
 {
 	enum raw1394_iso_disposition retval=RAW1394_ISO_OK;
+	int dropped_cycles;
 
 	freebob_connection_t *connection=(freebob_connection_t *) raw1394_get_userdata (handle);
 	assert(connection);
@@ -2125,12 +2138,14 @@ iso_master_receive_handler(raw1394handle_t handle, unsigned char *data,
 	struct iec61883_packet *packet = (struct iec61883_packet *) data;
 	assert(packet);
 	
-	// FIXME: dropped packets are very bad when transmitting and the other side is sync'ing on that!
-	connection->status.dropped+=dropped;
-
-#ifdef DEBUG	
-	connection->status.last_cycle=cycle;
-#endif
+	dropped_cycles = 0;
+	if (connection->status.last_cycle != cycle && connection->status.last_cycle != -1) {
+		dropped_cycles = diffCycles(cycle, connection->status.last_cycle) - 1;
+		if(dropped_cycles) {
+			connection->status.dropped+=dropped_cycles;
+		}
+	}
+	connection->status.last_cycle = cycle;
 
 	if((packet->fmt == 0x10) && (packet->fdf != 0xFF) && (packet->dbs>0) && (length>=2*sizeof(quadlet_t))) {
 		unsigned int nevents=((length / sizeof (quadlet_t)) - 2)/packet->dbs;
@@ -2200,6 +2215,8 @@ static enum raw1394_iso_disposition
 								  unsigned int dropped)
 {
 	enum raw1394_iso_disposition retval=RAW1394_ISO_OK;
+	int dropped_cycles;
+
 	/* slave receive is easy if you assume that the connections are synced
 	Synced connections have matched data rates, so just receiving and 
 	calling the rcv handler would suffice in this case. As the connection 
@@ -2213,13 +2230,14 @@ static enum raw1394_iso_disposition
 	struct iec61883_packet *packet = (struct iec61883_packet *) data;
 	assert(packet);
 	
-	// FIXME: dropped packets are very bad when transmitting and the other side is sync'ing on that!
-	//connection->status.packets+=dropped;
-	connection->status.dropped+=dropped;
-
-#ifdef DEBUG	
-	connection->status.last_cycle=cycle;
-#endif
+	dropped_cycles = 0;
+	if (connection->status.last_cycle != cycle && connection->status.last_cycle != -1) {
+		dropped_cycles = diffCycles(cycle, connection->status.last_cycle) - 1;
+		if(dropped_cycles) {
+			connection->status.dropped+=dropped_cycles;
+		}
+	}
+	connection->status.last_cycle = cycle;
 
 	if((packet->fmt == 0x10) && (packet->fdf != 0xFF) && (packet->dbs>0) && (length>=2*sizeof(quadlet_t))) {
 		unsigned int nevents=((length / sizeof (quadlet_t)) - 2)/packet->dbs;
@@ -2308,10 +2326,9 @@ iso_master_transmit_handler(raw1394handle_t handle,
 	int nevents = iec61883_cip_fill_header (handle, &connection->status.cip, packet);
 	int nsamples=0;
 	int bytes_read;
-	
-#ifdef DEBUG	
-	connection->status.last_cycle=cycle;
+	int dropped_cycles;
 
+#ifdef DEBUG	
 	if(packet->fdf != 0xFF) {
 		connection->status.fdf=packet->fdf;
 	}
@@ -2333,9 +2350,15 @@ iso_master_transmit_handler(raw1394handle_t handle,
 		}
 	}
 	
-	// dropped packets are very bad when transmitting and the other side is sync'ing on that!
-	connection->status.dropped += dropped;
-		
+	dropped_cycles = 0;
+	if (connection->status.last_cycle != cycle && connection->status.last_cycle != -1) {
+		dropped_cycles = diffCycles(cycle, connection->status.last_cycle) - 1;
+		if(dropped_cycles) {
+			connection->status.dropped+=dropped_cycles;
+		}
+	}
+	connection->status.last_cycle = cycle;
+	
 	if (nsamples > 0) {
 
 		assert(connection->spec.dimension == packet->dbs);
@@ -2407,10 +2430,9 @@ static enum raw1394_iso_disposition
 	int nevents = iec61883_cip_fill_header (handle, &connection->status.cip, packet);
 	int nsamples=0;
 	int bytes_read;
+	int dropped_cycles;
 
-#ifdef DEBUG	
-	connection->status.last_cycle=cycle;
-
+#ifdef DEBUG
 	if(packet->fdf != 0xFF) {
 		connection->status.fdf=packet->fdf;
 	}
@@ -2418,7 +2440,6 @@ static enum raw1394_iso_disposition
 
 	enum raw1394_iso_disposition retval = RAW1394_ISO_OK;
 
-
 	if (nevents > 0) {
 		nsamples = nevents;
 	}
@@ -2431,10 +2452,15 @@ static enum raw1394_iso_disposition
 		}
 	}
 	
-	// dropped packets are very bad when transmitting and the other side is sync'ing on that!
-	//connection->status.packets+=dropped;
-	connection->status.dropped += dropped;
-		
+	dropped_cycles = 0;
+	if (connection->status.last_cycle != cycle && connection->status.last_cycle != -1) {
+		dropped_cycles = diffCycles(cycle, connection->status.last_cycle) - 1;
+		if(dropped_cycles) {
+			connection->status.dropped+=dropped_cycles;
+		}
+	}
+	connection->status.last_cycle = cycle;
+	
 	if (nsamples > 0) {
 		int bytes_to_read=nsamples*sizeof(quadlet_t)*connection->spec.dimension;
 
@@ -2510,6 +2536,7 @@ static enum raw1394_iso_disposition
  * Decoders and encoders
  */
 #ifdef ENABLE_SSE
+#warning SSE build
 typedef float v4sf __attribute__ ((vector_size (16)));
 typedef int v4si __attribute__ ((vector_size (16)));
 typedef int v2si __attribute__ ((vector_size (8)));
@@ -2739,18 +2766,18 @@ freebob_encode_stream_to_events(freebob_connection_t *connection,
 				}
 				break;
 			case freebob_buffer_type_float:
-/*				for(j = 0; j < read; j += 1) { // decode max nsamples
-					// don't care for overflow
-					float v = *floatbuff * multiplier;  // v: -231 .. 231
-					unsigned int tmp = ((int)v);
-					*target_event = htonl((tmp >> 8) | 0x40000000);
-					
-					floatbuff++;
-					target_event += connection->spec.dimension;
-				}*/
 				j=0;
 				if(read>3) {
 					for (j = 0; j < read-3; j += 4) {
+						// clip
+						int s;
+						for(s = 0; s < 4; s += 1) {
+							float tmp=*(floatbuff+s);
+							if(tmp > 1.0) tmp = 1.0;
+							if(tmp < -1.0) tmp = -1.0;
+							*(floatbuff+s) = tmp;
+						}
+
 						asm("movups %[floatbuff], %%xmm0\n\t"
 								"mulps %[ssemult], %%xmm0\n\t"
 								"cvttps2pi %%xmm0, %[out1]\n\t"
@@ -2779,7 +2806,11 @@ freebob_encode_stream_to_events(freebob_connection_t *connection,
 				}
 				for(; j < read; ++j) {
 				// don't care for overflow
-					float v = *floatbuff * multiplier;  // v: -231 .. 231
+					float v = *floatbuff;
+					// clip
+					if(v > 1.0) v = 1.0;
+					if(v < -1.0) v = -1.0;
+					v = v * multiplier;  // v: -231 .. 231
 					unsigned int tmp = (int)v;
 					*target_event = htonl((tmp >> 8) | 0x40000000);
 		
@@ -2995,8 +3026,11 @@ freebob_encode_stream_to_events(freebob_connection_t *connection,
 				break;
 			case freebob_buffer_type_float:
 				for(j = 0; j < read; j += 1) { // decode max nsamples
-					// don't care for overflow
-					float v = *floatbuff * multiplier;  // v: -231 .. 231
+					float v = *floatbuff;
+					// clip
+					if(v > 1.0) v = 1.0;
+					if(v < -1.0) v = -1.0;
+					v = v * multiplier;  // v: -231 .. 231
 					unsigned int tmp = ((int)v);
 					*target_event = htonl((tmp >> 8) | 0x40000000);
 					
